#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
# 
# Copyright © 2007 Frederik Elwert

import os
import logging as _log
import optparse as _optparse
from xdg import BaseDirectory as _basedir

APP_NAME = 'genesis'
LOCALE_DIR = 'locale'
DESKTOP_FILE = 'genesis.desktop'
DESKTOP_DIR = 'bin'
DATA_DIR = 'Genesis'
AUTOSTART_DIR = 'autostart'
IMAGE_DIR = 'icons'
CONFIG_DIR = os.path.join(_basedir.xdg_config_home, APP_NAME)
# CONFIG_DIR must be created if not present
# but not when run as root (e.g. during install
if not os.path.isdir(CONFIG_DIR) and os.geteuid() != 0:
    os.makedirs(CONFIG_DIR)
LOG_LEVEL = _log.DEBUG
# Use /tmp for logging when run as root
if os.geteuid() == 0:
    import tempfile as _tempfile
    LOG_FILE = os.path.join(_tempfile.gettempdir(), APP_NAME+'.log')
else:
    LOG_FILE = os.path.join(CONFIG_DIR, APP_NAME+'.log')

_parser = _optparse.OptionParser()
_parser.add_option('-d', '--debug', 
                   action='store_const', const=_log.DEBUG,
                   dest='console_level', default=_log.WARNING,
                   help='print debugging messages')
_options, _args = _parser.parse_args()

CONSOLE_LEVEL = _options.console_level

_log.basicConfig(level=LOG_LEVEL,
                    format='%(asctime)s %(name)-14s %(levelname)-8s %(message)s',
                    filename=LOG_FILE,
                    filemode='w')
_handler = _log.StreamHandler()
_handler.setLevel(CONSOLE_LEVEL)
_formatter = _log.Formatter('%(name)-14s: %(levelname)-8s %(message)s')
_handler.setFormatter(_formatter)
_log.getLogger('').addHandler(_handler)

