#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
# 
# Copyright © 2007-2009 Frederik Elwert

import pygtk
pygtk.require('2.0')
import gtk
import gtk.glade
import locale
import gettext
import evolution

import os
import sys

from Genesis.syncevolution import SyncEvolution, ServerConfig
from Genesis.config import *

## Setup ##
# l10n
locale.setlocale(locale.LC_ALL, '')
for module in (gettext, gtk.glade):
    module.bindtextdomain(APP_NAME, LOCALE_DIR)
    module.textdomain(APP_NAME)
_ = gettext.gettext

class ServerAssistant(object):
    """
    The assistant to add a new synchronization server.
    
    """

    def __init__(self, caller=None):
        """
        :Parameters:
            - `caller`: The object that called the ServerAssistant.
              If ``None``, Server Assistant runs standalone and will
              call ``gtk.main_quit`` when done. Otherwise, it will call
              ``caller.build_server_menu`` to update the server list.
              
        """
        self.caller = caller
    
        # GTK/Glade setup
        self.wTree = gtk.glade.XML(os.path.join(DATA_DIR, 'settings.glade'), 'server_assistant')
        
        signals = {
            'on_server_assistant_destroy': self.on_quit,
            'on_server_assistant_cancel': self.on_quit,
            'on_server_assistant_prepare': self.on_prepare,
            'on_server_assistant_apply': self.on_apply,
            'on_server_assistant_close': self.on_quit,
            'check_page': self.check_page,
            'toggle_source': self.toggle_source,
        }

        self.wTree.signal_autoconnect(signals)
        
        self.window = self.wTree.get_widget('server_assistant')
        self.server_page = self.wTree.get_widget('server_page')
        self.server_table = self.wTree.get_widget('server_table')
        self.account_page = self.wTree.get_widget('account_page')
        self.sources_page = self.wTree.get_widget('sources_page')
        
        self.name_entry = self.wTree.get_widget('name_entry')
        self.address_box = self.wTree.get_widget('address_box')
        self.server_url = self.wTree.get_widget('server_url')
        
        self.username_entry = self.wTree.get_widget('username_entry')
        self.password_entry = self.wTree.get_widget('password_entry')
        self.proxy_entry = self.wTree.get_widget('proxy_entry')
        self.proxy_checkbox = self.wTree.get_widget('proxy_cb')
        
        self.source_keys = ['addressbook', 'calendar', 'memo', 'todo']
        self.source_table = self.wTree.get_widget('source_table')
        self.source_checkboxes = {}
        for key in self.source_keys:
            self.source_checkboxes[key] = self.wTree.get_widget(
                '%s_checkbutton' % key)
        
        # Config preparation
        # Match ``titled`` server names to standard ones for nicer looking
        # choices.
        self.templates = {}
        for template in SyncEvolution.list_templates().keys():
            self.templates[template.title()] = template
        self.custom_text = _('Custom Server')
        self.server_list = SyncEvolution.list_servers().keys()
        
        # Server selection box
        self.server_box = gtk.combo_box_new_text()
        self.server_box.show()
        self.server_box.connect('changed', self.on_server_select)
        for template in self.templates:
            self.server_box.append_text(template)
        self.server_box.append_text(self.custom_text)
        self.server_table.attach(self.server_box, 1, 2, 1, 2)
        
        # Synchronization sources
        # Test for python-evolution version, since the API changed in 0.0.4
        if evolution.__version__ >= (0,0,4):
            addressbooks = evolution.ebook.list_addressbooks()
            calendars = evolution.ecal.list_calendars()
            memos = evolution.ecal.list_memo_sources()
            tasks = evolution.ecal.list_task_sources()
        else:
            addressbooks = evolution.list_addressbooks()
            calendars = evolution.list_calendars()
            memos = evolution.list_memo_sources()
            tasks = evolution.list_task_sources()
        self.sources = []
        for source in (addressbooks, calendars, memos, tasks):
            self.sources.append([key for key, url in source])
        self.source_boxes = {}
        row = 0
        # If a custom server is used, ask for the uri of the
        # sync sources.
        self.source_entries = {}
        for key, source in zip(self.source_keys, self.sources):
            # Combo boxes for source selection
            cb = gtk.combo_box_new_text()
            cb.set_sensitive(False)
            cb.connect('changed', self.check_page)
            for entry in source:
                cb.append_text(entry)
            self.source_table.attach(cb, 1, 2, row, row+1,
                    gtk.FILL, gtk.FILL)
            self.source_boxes[key] = cb
            # Text Entry for source uri specification.
            e = gtk.Entry()
            e.set_sensitive(False)
            e.set_tooltip_text(_("The source URI for this source.\n It  is appended to the server's URL to identify the server's database.\n If no source URI is given, the default will be used."))
            self.source_entries[key] = e
            l = gtk.Label()
            l.set_label('URI:')
            b = gtk.HBox()
            b.add(l)
            b.add(e)
            b.show_all()
            self.source_table.attach(b,
                2, 3, row, row+1)
            row += 1
        
        self.window.show_all()
        
    def on_prepare(self, assistant, page):
        if page is self.account_page:
            name = self.name_entry.get_text()
            self.config.name = name
            server_url = self.server_url.get_text()
            if server_url:
                self.config.server_config['syncURL'] = server_url
        if page is self.sources_page:
            # Finish the server setup before adding sources
            username = self.username_entry.get_text()
            password = self.password_entry.get_text()
            use_proxy = self.proxy_checkbox.get_active()
            proxy_url = self.proxy_entry.get_text()
            self.config.server_config['username'] = username
            self.config.server_config['password'] = password
            if use_proxy:
                self.config.server_config['useProxy'] = 'T'
                self.config.server_config['proxyHost'] = proxy_url
            else:
                self.config.server_config['useProxy'] = 'F'
            # Set the source uris from the template
            for key in self.source_keys:
                source_uri = self.config.source_config[key].get('uri', '')
                self.source_entries[key].set_text(source_uri)
                
    def on_apply(self, widget):
        for source in self.source_keys:
            self.config.add_source(source)
            if self.source_checkboxes[source].get_active():
                self.config.source_config[source]['evolutionsource'] = \
                        self.source_boxes[source].get_active_text()
                self.config.source_config[source]['sync'] = 'two-way'
                # If a custom server is used, and a source uri is given,
                # set the source uri.
                if self.source_entries[source].get_text():
                    self.config.source_config[source]['uri'] = \
                        self.source_entries[source].get_text()
            # The new setup enables all available data sources. So we need to
            # disable unwanted sources explicitly.
            else:
                self.config.source_config[source]['sync'] = 'none'
        self.config.save()
            
    def on_server_select(self, widget):
        name = self.name_entry.get_text()
        server_type = self.server_box.get_active_text()
        if server_type == self.custom_text:
            template = 'default'
        else:
            template = self.templates[server_type]
        self.config = ServerConfig(name, template)
        self.config.load_template()
        self.server_url.set_text(self.config.server_config.get('syncURL', ''))
        self.check_page(widget)
        
    def check_page(self, widget):
        page_num = self.window.get_current_page()
        page = self.window.get_nth_page(page_num)
        complete = True
        if page == self.server_page:
            if not self.name_entry.get_text():
                complete = False
            if self.name_entry.get_text() in self.server_list:
                complete = False
            server = self.server_box.get_active_text()
            if not server:
                complete = False
            if server == self.custom_text \
                    and not self.server_url.get_text():
                complete = False
        if page == self.account_page:
            if not (self.username_entry.get_text()
                    and self.password_entry.get_text()):
                complete = False
            if self.proxy_checkbox.get_active() \
                    and not self.proxy_entry.get_text():
                complete = False
        if page == self.sources_page:
            for key in self.source_keys:
                if self.source_checkboxes[key].get_active()\
                        and not self.source_boxes[key].get_active_text():
                    complete = False
        self.window.set_page_complete(page, complete)
        
    def toggle_source(self, widget):
        for key in self.source_keys:
            if self.source_checkboxes[key].get_active():
                sensitive = True
            else:
                sensitive = False
            self.source_boxes[key].set_sensitive(sensitive)
            self.source_entries[key].set_sensitive(sensitive)
        self.check_page(None)
            
    def on_quit(self, widget):
        if self.caller:
            self.window.destroy()
            self.caller.build_server_menu()
        else:
            gtk.main_quit()
        

if __name__ == '__main__':
    sa = ServerAssistant()
    gtk.main()
    #sc = ServerConfig('scheduleworld')
    #sys.exit()
    
